<?php
/*
Plugin Name: MW Latest Tweet
Plugin URI: http://www.meyerweb.com/eric/tools/wordpress/latesttweet/
Description: Puts the latest tweet from wherever you call it, with cached tweets for backup and a few options to customize what comes back.
Version: 1.0
Author: Eric A. Meyer
Author URI: http://meyerweb.com/
*/

register_activation_hook(__FILE__,'_mw_lt_setup');
add_action('admin_menu', '_mw_lt_add_admin',1);
add_action('admin_head', '_mw_lt_add_styles',13);



FUNCTION _mw_lt_update($show=true) {
	$data = get_option("mw_lt_data");
	$options = get_option("mw_lt_options");

	if (!$options->username) return;

	$expire = $data->cached + ($options->cachefor_min * 60);
	if (time() > $expire) {
		$url = "http://twitter.com/statuses/user_timeline/" . $options->username . ".json";
		$result = _mw_lt_json_parse(_mw_lt_get_stream($url));
		if (!$options->showreplies) $result = _mw_lt_drop_replies($result);
		if ($result) {
			$data->last_tweet = $result[0];
			$data->cached = time();
			update_option("mw_lt_data",$data);
		}
	}
	if ($show) _mw_lt_display_html($data->last_tweet);
}


FUNCTION _mw_lt_display_html($tweet) {
	$options = get_option("mw_lt_options");
	if ($options->humantime) $tweet->created_at = _mw_lt_human_time($tweet->created_at,2,true);
	if ($options->htmlpattern) {
		$output = $options->htmlpattern;
		$patterns = array('	/%%TEXT%%/',
						'/%%CREATED_AT%%/',
						'/%%ID%%/',
						'/%%SOURCE%%/',
						'/%%IN_REPLY_TO_STATUS_ID%%/',
						'/%%IN_REPLY_TO_USER_ID%%/',
						'/%%USER_NAME%%/',
						'/%%USER_SCREEN_NAME%%/',
						'/%%USER_FOLLOWERS_COUNT%%/',
						'/%%USER_PROFILE_IMAGE_URL%%/',
						'/%%USER_DESCRIPTION%%/',
						'/%%USER_URL%%/',
						'/%%USER_LOCATION%%/',
						'/%%USER_ID%%/'
						);
		$replaces = array(	$tweet->text,
						$tweet->created_at,
						$tweet->id,
						$tweet->source,
						$tweet->in_reply_to_status_id,
						$tweet->in_reply_to_user_id,
						$tweet->user->name,
						$tweet->user->screen_name,
						$tweet->user->followers_count,
						$tweet->user->profile_image_url,
						$tweet->user->description,
						$tweet->user->url,
						$tweet->user->location,
						$tweet->user->id
						);
		$output = preg_replace($patterns, $replaces, $output);
	} else {
		$output = $tweet->text;
	}
	if ($output) echo $output;
}

FUNCTION _mw_lt_human_time($time,$bits=1,$weeks=false) {
	$timestamp = _mw_lt_fix_twitter_time($time);
	if (!$timestamp || $timestamp < 0) $timestamp = $time;
	$humanized = '';
	$now = time();
	$interval = abs($now - $timestamp);

	if ($interval < 60) {
		$humanized = $interval . ' seconds';
	}
	if ($interval > 604800 && $weeks) {
		$weeks = floor($interval / 604800);
		$humanized .= $weeks . ' week';
		if ($weeks > 1) $humanized .= "s";
		$humanized .=", ";
		$interval = $interval % 604800;
	}
	if ($interval > 86400) {
		$days = floor($interval / 86400);
		$humanized .= $days . ' day';
		if ($days > 1) $humanized .= "s";
		$humanized .=", ";
		$interval = $interval % 86400;
	}
	if ($interval > 3600) {
		$hours = floor($interval / 3600);
		$humanized .= $hours . ' hour';
		if ($hours> 1 ) $humanized .= "s";
		$humanized .=", ";
		$interval = $interval % 3600;
	}
	if ($interval > 60) {
		$minutes = floor($interval / 60);
		$humanized .= $minutes . ' minute';
		if ($minutes > 1) $humanized .= "s";
	}

	if ($humanized) {
		$temp = explode(', ',$humanized);
		$humanized = '';
		for ($i = 0; $i < $bits; $i++) {
			if ($temp[$i]) $humanized .= $temp[$i] . ', ';
		}
		$humanized = substr($humanized,0,-2);
		if ($now > $timestamp) {
			$humanized .= ' ago';
		} else {
			$humanized = 'in ' . $humanized;
		}
		if ($now == $timestamp) {
			$humanized = "now";
		}
		return $humanized;
	} else {
		return $time;
	}
}

FUNCTION _mw_lt_fix_twitter_time($time) {
	return strtotime(
		preg_replace(	'/(...) (... \d\d) (\d\d:\d\d:\d\d) (\+\d\d\d\d) (\d\d\d\d)/',
					'\1, \2 \5 \3 \4',
					$time));
}

// http://www.sleepydisco.co.uk/plugins/simpletwitter/
FUNCTION _mw_lt_get_stream($url) {
	if (FUNCTION_exists('curl_init') && $url) {	
		$session = curl_init($url);
		curl_setopt($session, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($session, CURLOPT_FAILONERROR, true);
		curl_setopt($session, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt($session, CURLOPT_TIMEOUT, 10);
		$stream = curl_exec($session);
		curl_close($session);
	}
	return $stream;
}

// http://www.php.net/manual/en/FUNCTION.json-decode.php#80606
FUNCTION _mw_lt_json_parse($stream) {
	if (function_exists('json_decode')) {
		$result = json_decode($stream, true);
	} else {
		if (!class_exists('Services_JSON')) require_once("mw_latest_tweet/json.php");
		$parser = new Services_JSON();
		$result = $parser->decode($stream);
	}
	return $result;
}

// http://www.php.net/manual/en/FUNCTION.array-filter.php#24477
FUNCTION _mw_lt_drop_replies($array) {
	if (!$array) return;
	$thisarray = array ();
	foreach($array as $value)
		if($value->in_reply_to_user_id == false)
			$thisarray[] = $value;
	return $thisarray;
}



FUNCTION _mw_lt_add_admin() {
    if (function_exists('add_options_page')) {
		add_options_page('Latest Tweet', 'Latest Tweet', 10, 'mw_latest_tweet', '_mw_lt_admin_page');
    }
}

FUNCTION _mw_lt_add_styles() {
	if ($_GET['page'] == 'mw_latest_tweet') {
		$stylePath = get_settings('siteurl') . '/wp-content/plugins';
		echo '<link rel="stylesheet" type="text/css" href="' . $stylePath . '/mw_latest_tweet/mw_lt_admin.css" />';
	}
}

FUNCTION _mw_lt_setup($options, $data) {
	if (!$options || $options == NULL) {
		$options->showreplies = true;
		$options->humantime = true;
		$options->cachefor_min = 15;
		$options->username = '';
		$options->htmlpattern = '%%TEXT%%';
		update_option("mw_lt_options",$options);
	}
	if (!$data || $data == NULL) {
		$data->cached = 1;
		update_option("mw_lt_data",$data);
	}
}

FUNCTION _mw_lt_admin_page() {
	$options = get_option("mw_lt_options");
	$data = get_option("mw_lt_data");
	_mw_lt_setup($options, $data);

	echo '<div class="wrap"><h2>Latest Tweet</h2>';
	if (!function_exists('curl_init')) {
		echo <<<END
<div class="error">
<p>
This plugin requires that cURL be installed on your server and available via PHP.  One or both of these appear to be lacking.  Please contact your server's administrator (assuming that's someone other than you) to rectify the situation.  Until that happens, no tweets will be downloaded, cached, or displayed by this plugin.  Sorry!
</p>
</div>
END;
	} else {
		if (isset($_POST['update'])) {
			$options->htmlpattern = html_entity_decode(stripslashes($_POST['html']));
			$options->showreplies = $_POST['replies'];
			$options->humantime = $_POST['human'];
			if ($_POST['cache']) $options->cachefor_min = $_POST['cache'];
			if ($_POST['user']) $options->username = $_POST['user'];
			update_option("mw_lt_options",$options);
			echo '<p id="message" class="updated">Options updated.</p>';
		}
		$options = get_option("mw_lt_options");
		$rTime = time() - rand(0,86400);
?>


<form method="post">

<fieldset class="options">
<legend><?php _e('Options') ?></legend>
<table cellspacing="0" class="mw_options"> 
<tbody>
<tr>
<th scope="row">
<label for="user"><?php _e('Your Twitter screen name'); ?></label>
</th> 
<td>
<tt>http://twitter.com/</tt><input type="text" name="user" id="user" value="<?php echo $options->username; ?>"/> <?php if ($options->username) {echo '<small>(<a href="http://twitter.com/' . $options->username . '">go there</a>)</small>';} ?>
</td> 
</tr>
<tr>
<th scope="row">
<label for="cache"><?php _e('Cache latest tweet for'); ?></label>
</th>
<td>
<input type="text" name="cache" id="cache" size="5" maxlength="5" value="<?php echo $options->cachefor_min ?>"/> <?php _e('minutes') ?> <small>(minimum 1, maximum <abbr title="which is 69 days, 10 hours, 39 minutes">99999</abbr>;
<?php
if ($data->cached < time()) {
	_mw_lt_update(0);
	$data = get_option('mw_lt_data');
}
echo "last cached ";
if ($options->humantime) {
	echo _mw_lt_human_time($data->cached,2);
} else {
	echo date("r", $data->cached);
}
?>
)</small>
</td>
</tr> 
<tr>
<th scope="row">
<?php _e('Use "human" time intervals?'); ?>
</th>
<td>
<label for="human-yes"><input type="radio" name="human" id="human-yes" value="1" <?php if ($options->humantime) {echo 'checked="checked" ';} ?>/> <?php _e('Yes') ?> <small>(e.g., <?php echo _mw_lt_human_time($rTime,2); ?>)</small></label>
<label for="human-no"><input type="radio" name="human" id="human-no" value="0" <?php if (!$options->humantime) {echo 'checked="checked" ';} ?>/> <?php _e('No') ?> <small>(e.g., <?php echo date('D M d H:i:s +0000 Y',$rTime + (date("Z",$rTime))); ?>)</small></label>
</td>
</tr>
<tr>
<th scope="row">
<?php _e('Include @replies?'); ?>
</th>
<td>
<label for="replies-yes"><input type="radio" name="replies" id="replies-yes" value="1" <?php if ($options->showreplies) {echo 'checked="checked" ';} ?>/> <?php _e('Yes') ?></label>
<label for="replies-no"><input type="radio" name="replies" id="replies-no" value="0" <?php if (!$options->showreplies) {echo 'checked="checked" ';} ?>/> <?php _e('No') ?></label>
</td>
</tr>
<tr>
<th scope="row">
<?php _e('XHTML output'); ?>
<p>Emptying this field will cause just the bare text of the tweet to be returned; eqivalent to <code>%%TEXT%%</code>.</p>
</th>
<td>
<textarea name="html" id="html" rows="8" cols="60"><?php echo htmlentities($options->htmlpattern); ?></textarea>
<p class="mw_htmlstrings">

Recognized replacement patterns are:
<span>
<code>%%TEXT%%</code> 
<code>%%CREATED_AT%%</code> 
<code>%%ID%%</code> 
<code>%%SOURCE%%</code> 
<code>%%IN_REPLY_TO_STATUS_ID%%</code> 
<code>%%IN_REPLY_TO_USER_ID%%</code> 
<code>%%USER_NAME%%</code> 
<code>%%USER_SCREEN_NAME%%</code> 
<code>%%USER_FOLLOWERS_COUNT%%</code> 
<code>%%USER_PROFILE_IMAGE_URL%%</code> 
<code>%%USER_DESCRIPTION%%</code> 
<code>%%USER_URL%%</code> 
<code>%%USER_LOCATION%%</code> 
<code>%%USER_ID%%</code>
</span>
<a href="http://meyerweb.com/eric/tools/wordpress/mw_latest_tweet.html">Please see the plugin documentation</a> for details.
</p>
</td>
</tr>
</tbody>
</table> 


<p class="submit">
<input type="submit" name="update" id="update" value="<?php _e('Update') ?>" />
</p>
</fieldset>

</form>

</div>

<?php
	}
}

?>